/**
 * DevExtreme (esm/ui/date_range_box/strategy/rangeCalendar.js)
 * Version: 23.1.1 (build 23114-0308)
 * Build date: Mon Apr 24 2023
 *
 * Copyright (c) 2012 - 2023 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
import $ from "../../../core/renderer";
import CalendarStrategy from "../../date_box/ui.date_box.strategy.calendar";
import {
    extend
} from "../../../core/utils/extend";
var CALENDAR_RANGE_START_DATE_CLASS = "dx-calendar-range-start-date";
var CALENDAR_RANGE_END_DATE_CLASS = "dx-calendar-range-end-date";
class RangeCalendarStrategy extends CalendarStrategy {
    constructor(dateBox) {
        super();
        this.dateBox = dateBox;
        this.dateRangeBox = dateBox.option("_dateRangeBoxInstance")
    }
    popupConfig(popupConfig) {
        return extend(true, super.popupConfig(popupConfig), {
            position: {
                of: this.dateRangeBox.$element()
            }
        })
    }
    _getWidgetOptions() {
        return extend(super._getWidgetOptions(), {
            values: this.dateRangeBox.option("value"),
            selectionMode: "range",
            viewsCount: 2,
            width: 260
        })
    }
    getValue() {
        return this._widget.option("values")
    }
    _updateValue() {
        if (!this._widget) {
            return
        }
        this._widget.option("values", this.dateRangeBox.option("value"))
    }
    _valueChangedHandler(_ref) {
        var {
            value: value,
            previousValue: previousValue,
            event: event
        } = _ref;
        this.tryUpdateValue(event, 0, value, previousValue);
        this.tryUpdateValue(event, 1, value, previousValue)
    }
    tryUpdateValue(event, index, value, previousValue) {
        if (this.getDateRangeBox()._isSameDates(value[index], previousValue[index])) {
            return
        }
        if ("instantly" === this.dateBox.option("applyValueMode")) {
            this.dateBoxValue(this.getValue()[index], event)
        }
    }
    dateBoxValue() {
        if (arguments.length) {
            return this.dateBox.dateValue.apply(this.dateBox, arguments)
        } else {
            return this.dateBox.dateOption.apply(this.dateBox, ["value"])
        }
    }
    _cellClickHandler(_ref2) {
        var {
            value: value,
            event: event
        } = _ref2;
        if ("instantly" === this.dateBox.option("applyValueMode")) {
            if (this.isStartDateSelected(event)) {
                this.setActiveStartDateBox();
                this.dateBoxValue(value, event);
                this.getDateRangeBox().getEndDateBox().focus();
                this.setActiveEndDateBox()
            } else {
                this.dateBoxValue(value, event);
                this.getDateRangeBox().close()
            }
        }
    }
    setActiveStartDateBox() {
        this.dateBox = this.dateRangeBox.getStartDateBox()
    }
    setActiveEndDateBox() {
        this.dateBox = this.dateRangeBox.getEndDateBox()
    }
    getDateRangeBox() {
        return this.dateRangeBox
    }
    isStartDateSelected(_ref3) {
        var {
            currentTarget: currentTarget
        } = _ref3;
        if ($(currentTarget).hasClass(CALENDAR_RANGE_START_DATE_CLASS)) {
            return true
        }
        return false
    }
    isEndDateSelected(_ref4) {
        var {
            currentTarget: currentTarget
        } = _ref4;
        if ($(currentTarget).hasClass(CALENDAR_RANGE_END_DATE_CLASS)) {
            return true
        }
        return false
    }
}
export default RangeCalendarStrategy;
